/*********************************************************************
  This is an example for our nRF51822 based Bluefruit LE modules

  Pick one up today in the adafruit shop!

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Macetech LED Shades from Macetech LLC, code/guide by John Wall for Adafruit

  MIT license, check LICENSE for more information
  All text above, and the splash screen below must be included in
  any redistribution
*********************************************************************/

#include <string.h>
#include <Arduino.h>
#include <SPI.h>
#if not defined (_VARIANT_ARDUINO_DUE_X_) && not defined (_VARIANT_ARDUINO_ZERO_)
#include <SoftwareSerial.h>
#endif

#include "Adafruit_BLE.h"
#include "Adafruit_BluefruitLE_SPI.h"
#include "Adafruit_BluefruitLE_UART.h"

#include "BluefruitConfig.h"

#include <Adafruit_NeoPixel.h>

/*=========================================================================
    APPLICATION SETTINGS

      FACTORYRESET_ENABLE       Perform a factory reset when running this sketch
     
                                Enabling this will put your Bluefruit LE module
                              in a 'known good' state and clear any config
                              data set in previous sketches or projects, so
                                running this at least once is a good idea.
     
                                When deploying your project, however, you will
                              want to disable factory reset by setting this
                              value to 0.  If you are making changes to your
                                Bluefruit LE device via AT commands, and those
                              changes aren't persisting across resets, this
                              is the reason why.  Factory reset will erase
                              the non-volatile memory where config data is
                              stored, setting it back to factory default
                              values.
         
                                Some sketches that require you to bond to a
                              central device (HID mouse, keyboard, etc.)
                              won't work at all with this feature enabled
                              since the factory reset will clear all of the
                              bonding data stored on the chip, meaning the
                              central device won't be able to reconnect.
    PIN                       Which pin on the Arduino is connected to the NeoPixels?
    NUMPIXELS                 How many NeoPixels are attached to the Arduino?
    -----------------------------------------------------------------------*/
#define FACTORYRESET_ENABLE     1

#define PIN                     6
#define NUMPIXELS               68
/*=========================================================================*/

Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUMPIXELS, PIN);

// Create the bluefruit object, either software serial...uncomment these lines
/*
  SoftwareSerial bluefruitSS = SoftwareSerial(BLUEFRUIT_SWUART_TXD_PIN, BLUEFRUIT_SWUART_RXD_PIN);

  Adafruit_BluefruitLE_UART ble(bluefruitSS, BLUEFRUIT_UART_MODE_PIN,
                      BLUEFRUIT_UART_CTS_PIN, BLUEFRUIT_UART_RTS_PIN);
*/

/* ...or hardware serial, which does not need the RTS/CTS pins. Uncomment this line */
// Adafruit_BluefruitLE_UART ble(BLUEFRUIT_HWSERIAL_NAME, BLUEFRUIT_UART_MODE_PIN);

/* ...hardware SPI, using SCK/MOSI/MISO hardware SPI pins and then user selected CS/IRQ/RST */
Adafruit_BluefruitLE_SPI ble(BLUEFRUIT_SPI_CS, BLUEFRUIT_SPI_IRQ, BLUEFRUIT_SPI_RST);

/* ...software SPI, using SCK/MOSI/MISO user-defined SPI pins and then user selected CS/IRQ/RST */
//Adafruit_BluefruitLE_SPI ble(BLUEFRUIT_SPI_SCK, BLUEFRUIT_SPI_MISO,
//                             BLUEFRUIT_SPI_MOSI, BLUEFRUIT_SPI_CS,
//                             BLUEFRUIT_SPI_IRQ, BLUEFRUIT_SPI_RST);


// A small helper
void error(const __FlashStringHelper*err) {
  Serial.println(err);
  while (1);
}

// function prototypes over in packetparser.cpp
uint8_t readPacket(Adafruit_BLE *ble, uint16_t timeout);
float parsefloat(uint8_t *buffer);
void printHex(const uint8_t * data, const uint32_t numBytes);

// the packet buffer
extern uint8_t packetbuffer[];

const int sampleWindow = 50; // Sample window width for the mic in mS (50 mS = 20Hz)
unsigned int sample;
int maximum = 600;

uint8_t red = 0;// Mode and color variables
uint8_t green = 0;
uint8_t blue = 0;
uint8_t lastR = 0;
uint8_t lastG = 0;
uint8_t lastB = 0;
byte mode = 1;
byte button = 0;
int identifier = 0;
byte fadeDelay = 35;
byte color = 0;
byte rainbowDelay = 20;
byte colorAdvance = 0;
unsigned long previousRainbowMillis = 0;
int column;
int row;

int ledArray[16][5] = // Because the Neopixels are in a chain, to access
{ // their adresses in a grid we use a 2D array. Nifty!
  { -1, 14, 43, 44, -1},
  {13, 15, 42, 45, 67},
  {12, 16, 41, 46, 66},
  {11, 17, 40, 47, 65},
  {10, 18, 39, 48, 64},
  { 9, 19, 38, 49, 63},
  { 8, 20, 37, 50, -1},
  { 7, 21, -1, -1, -1},
  { 6, 22, -1, -1, -1},
  { 5, 23, 36, 51, -1},
  { 4, 24, 35, 52, 62},
  { 3, 25, 34, 53, 61},
  { 2, 26, 33, 54, 60},
  { 1, 27, 32, 55, 59},
  { 0, 28, 31, 56, 58},
  { -1, 29, 30, 57, -1}
};

int levelsArray[16] = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
byte clr; // This array stores sound intensity levels for the 4th mode

/**************************************************************************/
/*!
    @brief  Sets up the HW an the BLE module (this function is called
            automatically on startup)
*/
/**************************************************************************/
void setup(void)
{
  strip.begin(); // This initializes the NeoPixel library.

  Serial.begin(115200);
  Serial.println(F("Adafruit Bluefruit Connected Macetech LED Shades"));
  Serial.println(F("------------------------------------------------"));

  /* Initialise the module */
  Serial.print(F("Initialising the Bluefruit LE module: "));

  if ( !ble.begin(VERBOSE_MODE) )
  {
    error(F("Couldn't find Bluefruit, make sure it's in CoMmanD mode & check wiring?"));
  }
  Serial.println( F("OK!") );

  if ( FACTORYRESET_ENABLE )
  {
    /* Perform a factory reset to make sure everything is in a known state */
    Serial.println(F("Performing a factory reset: "));
    if ( ! ble.factoryReset() ) {
      error(F("Couldn't factory reset"));
    }
  }

  /* Disable command echo from Bluefruit */
  ble.echo(false);

  Serial.println("Requesting Bluefruit info:");
  /* Print Bluefruit information */
  ble.info();

  Serial.println(F("Please use Adafruit Bluefruit LE app to connect in Color Picker or Controller mode"));

  ble.verbose(false);  // debug info is a little annoying after this point!

  for (uint8_t i = 0; i < NUMPIXELS; i++) {
    strip.setPixelColor(i, strip.Color(0, 0, 0)); // off
  }
  strip.show();
  strip.setBrightness(26);// I'd leave this where it is, 26 is pleeenty bright and saves power

  /* Wait for connection */
  while (! ble.isConnected()) {
    delay(500);
  }

  Serial.println(F("***********************"));

  // Set Bluefruit to DATA mode
  Serial.println( F("Switching to DATA mode!") );
  ble.setMode(BLUEFRUIT_MODE_DATA);

  Serial.println(F("***********************"));

}

/**************************************************************************/
/*!
    @brief  Constantly poll for new command or response data
*/
/**************************************************************************/
void loop(void)
{
  /* Wait for new data to arrive */
  uint8_t len = readPacket(&ble, 1);
  //if (len == 0) return;
  /* Got a packet! */
  printHex(packetbuffer, len);

  if (packetbuffer[1] == 'C')// If the packet is from the color picker, fill the shades with it
  {
    mode = 1;
    red = packetbuffer[2];
    green = packetbuffer[3];
    blue = packetbuffer[4];
    Serial.println ("RGB #");
    if (red < 0x10) Serial.print("0");
    Serial.println(red);
    if (green < 0x10) Serial.print("0");
    Serial.println(green);
    if (blue < 0x10) Serial.print("0");
    Serial.println(blue);

    for (uint16_t i = 0; i < strip.numPixels(); i++)
    {
      strip.setPixelColor(i, strip.Color(red, green, blue));
      strip.show();
      delay(20);
    }
  }

  else if (packetbuffer[1] == 'B')// If a controller button is pressed, read it and assign the mode
  {
    uint8_t buttnum = packetbuffer[2] - '0';
    boolean pressed = packetbuffer[3] - '0';
    button = buttnum;
    mode = 2;
  }

  // Color

  if (mode == 2)// If a button was pressed
  {
    if (button == 1)// First button is a single color fading rainbow
    {
      for (uint16_t i = 0; i < strip.numPixels(); i++) {
        strip.setPixelColor(i, Wheel(color));
      }
      strip.show();

      color++;
      if (color > 255) color = 0;
      delay(fadeDelay);
    }

    else if (button == 2)// Second is a ful spectrum rainbow
    {
      unsigned long currentRainbowMillis = millis();

      if (currentRainbowMillis - previousRainbowMillis >= rainbowDelay) {

        colorAdvance++;

        if (colorAdvance > 255) colorAdvance = 0;

        for (int column = 0; column < 16; column++)
        {
          for (int row = 0; row < 5; row++)
          {
            int led = ledArray[column][row];

            if (led != -1)
            {
              strip.setPixelColor(led, Wheel(map(column, 15, 0, 0, 255) - colorAdvance));
            }
          }
        }
        strip.show();
        previousRainbowMillis = currentRainbowMillis;
      }
    }

    else if (button == 3)// Third is a rainbow lelft to right VU meter
    {
      unsigned long startMillis = millis(); // Start of sample window
      unsigned int peakToPeak = 0;   // peak-to-peak level

      unsigned int signalMax = 0;
      unsigned int signalMin = 400;

      // collect data for 50 mS
      while (millis() - startMillis < sampleWindow)
      {
        sample = analogRead(0);

        if (sample < 1024)  // toss out spurious readings
        {
          if (sample > signalMax)
          {
            signalMax = sample;  // save just the max levels
          }
          else if (sample < signalMin)
          {
            signalMin = sample;  // save just the min levels
          }
        }
      }

      peakToPeak = signalMax - signalMin - 15;  // max - min = peak-peak amplitude
      if (peakToPeak < 0) peakToPeak = 0;
      else if (peakToPeak > 1023) peakToPeak = 0;

      int level = map(peakToPeak, 0, maximum, -1, 16);

      Serial.println(peakToPeak);

      for (int column = 0; column <= level; column++)
      {
        for (int row = 0; row < 5; row++)
        {
          int ledOn = ledArray[15 - column][row];

          if (ledOn != -1)
          {
            strip.setPixelColor(ledOn, Wheel(map(column, 0, 15, 0, 255)));
          }
        }
      }

      for (int column = level + 1; column < 16; column++)
      {
        for (int row = 0; row < 5; row++)
        {
          int ledOff = ledArray[15 - column][row];

          if (ledOff != -1)
          {
            strip.setPixelColor(ledOff, strip.Color(0, 0, 0));
          }
        }
      }

      strip.show();
    }

    else if (button == 4)// Fourth is a single color rainbow left to right VU graph
    {
      unsigned long startMillis = millis(); // Start of sample window
      unsigned int peakToPeak = 0;   // Peak-to-peak level

      unsigned int signalMax = 0;
      unsigned int signalMin = 400;

      // Collect data for 50 mS
      while (millis() - startMillis < sampleWindow)
      {
        sample = analogRead(0);
        if (sample < 1024)  // Toss out spurious readings
        {
          if (sample > signalMax)
          {
            signalMax = sample;  // Save just the max levels
          }
          else if (sample < signalMin)
          {
            signalMin = sample;  // Save just the min levels
          }
        }
      }

      peakToPeak = signalMax - signalMin - 15;  // Max - min = peak-peak amplitude
      if (peakToPeak < 0) peakToPeak = 0;
      else if (peakToPeak > 1023) peakToPeak = 0;

      int level = map(peakToPeak, 0, maximum, -1, 5);

      for (int i = 0; i < 16; i++) {
        levelsArray[i] = levelsArray[i + 1];
      }

      levelsArray[15] = level;

      for (int column = 0; column < 16; column++)
      {
        for (int row = 0; row <= levelsArray[column]; row++)
        {
          int ledOn = ledArray[column][row];

          if (ledOn != -1)
          {
            strip.setPixelColor(ledOn, Wheel(clr));
          }
        }

        for (int row = levelsArray[column] + 1; row < 5; row++)
        {
          int ledOff = ledArray[column][row];

          if (ledOff != -1)
          {
            strip.setPixelColor(ledOff, strip.Color(0, 0, 0)); //Wheel(clr+128));//
          }
        }
      }

      strip.show();
      clr++;
      if (clr > 255) clr = 0;
    }
  }
}

void rainbow(uint8_t wait)
{
  uint16_t i, j;

  for (j = 0; j < 256; j++)
  {
    for (i = 0; i < strip.numPixels(); i++)
    {
      strip.setPixelColor(i, Wheel((i + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait)
{
  uint16_t i, j;

  for (j = 0; j < 256 * 5; j++)
  { // 5 cycles of all colors on wheel
    for (i = 0; i < strip.numPixels(); i++)
    {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait)
{
  for (int j = 0; j < 10; j++)
  { //do 10 cycles of chasing
    for (int q = 0; q < 3; q++)
    {
      for (int i = 0; i < strip.numPixels(); i = i + 3)
      {
        strip.setPixelColor(i + q, c);// Turn every third pixel on
      }
      strip.show();

      delay(wait);

      for (int i = 0; i < strip.numPixels(); i = i + 3)
      {
        strip.setPixelColor(i + q, 0);// Turn every third pixel off
      }
    }
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos)
{
  WheelPos = 255 - WheelPos;
  if (WheelPos < 85)
  {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else if (WheelPos < 170)
  {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  } else
  {
    WheelPos -= 170;
    return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  }
}
